#ifndef MODULE_DIR_H
#define MODULE_DIR_H

struct module {
	char *path, *name;

	void *handle;
	void (*init)(struct module *module);
	void (*deinit)(void);

	bool initialized:1;

        struct module *next;
};

/* Define a statically linked module by supplying its name string, init and deinit functions. This first checks that the module is wanted (module_names is NULL or else includes name) and isn't already loaded in the given modules linked list. It then constructs a struct module and appends it to the list. */
void module_load_static(struct module **modules, const char **module_names, const char *name, void (*init_func)(struct module *), void (*deinit_func)(void));
/* Split a user-configured module names list into a temporary array, converting to base form and dropping duplicates, suitable for use as the module_names argument to module_load_static. */
const char **module_parse_names(const char *module_names);

/* Load modules of a given type (called a "module directory" due to the prior dlopen based implementation). mod_names is a space separated list of module names to load, or NULL for all available. Each function goes in a separate file so that each module collection links into only the binaries that require it. */
void module_dir_load_adm(struct module **modules, const char *mod_names);
void module_dir_load_auth(struct module **modules, const char *mod_names);
void module_dir_load_mail(struct module **modules, const char *mod_names);
void module_dir_load_old_stats(struct module **modules, const char *mod_names);
void module_dir_load_settings(struct module **modules, const char *mod_names);
void module_dir_load_ssl_iostream(struct module **modules, const char *mod_names);

/* Call init() in all modules */
void module_dir_init(struct module *modules);
/* Call deinit() in all modules and mark them NULL so module_dir_unload()
   won't do it again. */
void module_dir_deinit(struct module *modules);
/* Unload all modules */
void module_dir_unload(struct module **modules);
/* Find a module by name. */
struct module *module_dir_find(struct module *modules, const char *name);

/* Former dlsym wrapper, left as stub until its last use can be eliminated (config/config-parser.c) */
void *module_get_symbol_quiet(struct module *module, const char *symbol);

/* Returns module's name without "_plugin" suffix. */
const char *module_get_plugin_name(struct module *module);

#endif
