/* Copyright (c) 2011-2018 Dovecot authors, see the included COPYING file */

#include "lib.h"
#include "time-util.h"
#include "stats-plugin.h"
#include "mail-stats.h"

#include <sys/resource.h>

static void
user_trans_stats_get(struct stats_user *suser, struct mail_stats *dest_r)
{
	struct stats_transaction_context *strans;

	mail_stats_add_transaction(dest_r, &suser->finished_transaction_stats);
	for (strans = suser->transactions; strans != NULL; strans = strans->next)
		mail_stats_add_transaction(dest_r, &strans->trans->stats);
}

void mail_stats_fill(struct stats_user *suser, struct mail_stats *stats_r)
{
	static bool getrusage_broken = FALSE;
	static struct rusage prev_usage;
	struct rusage usage;

	i_zero(stats_r);
	/* cputime */
	if (getrusage(RUSAGE_SELF, &usage) < 0) {
		if (!getrusage_broken) {
			i_error("getrusage() failed: %m");
			getrusage_broken = TRUE;
		}
		usage = prev_usage;
	} else if (timeval_diff_usecs(&usage.ru_stime, &prev_usage.ru_stime) < 0) {
		/* This seems to be a Linux bug. */
		usage.ru_stime = prev_usage.ru_stime;
	}
	prev_usage = usage;

	stats_r->user_cpu = usage.ru_utime;
	stats_r->sys_cpu = usage.ru_stime;
	stats_r->min_faults = usage.ru_minflt;
	stats_r->maj_faults = usage.ru_majflt;
	stats_r->vol_cs = usage.ru_nvcsw;
	stats_r->invol_cs = usage.ru_nivcsw;
	stats_r->disk_input = (unsigned long long)usage.ru_inblock * 512ULL;
	stats_r->disk_output = (unsigned long long)usage.ru_oublock * 512ULL;
	i_gettimeofday(&stats_r->clock_time);
	user_trans_stats_get(suser, stats_r);
}
